@@ -35,12 +35,12 @@ sub track_unmemoize { # after generate_setup
 	return unless $config->{memoize};
 	map{package Audio::Nama::Track; unmemoize ($_)} @track_functions;
 }
-sub rememoize {
+sub restart_wav_memoize {
 	return unless $config->{memoize};
 	map{package Audio::Nama::Wav; unmemoize ($_); memoize($_) } 
 		@wav_functions;
 }
-sub init_memoize {
+sub init_wav_memoize {
 	return unless $config->{memoize};
 	map{package Audio::Nama::Wav; memoize($_) } @wav_functions;
 }
