@@ -36,11 +36,18 @@ sub revise_prompt {
     $text->{term}->callback_handler_install($override//prompt(), \&process_line);
 }
 }
+
+	
 sub prompt {
-	"nama [". ($this_bus eq 'Main' ? '': "$this_bus/").  
+
+		git_branch_display(). "nama [". ($this_bus eq 'Main' ? '': "$this_bus/").  
 		($this_track ? $this_track->name : '') . "] ('h' for help)> "
 }
-sub check_for_spacebar_hit {
+sub detect_spacebar {
+
+	# create a STDIN watcher to intervene when space
+	# received in column one
+	
 	$engine->{events}->{stdin} = AE::io(*STDIN, 0, sub {
 		&{$text->{term_attribs}->{'callback_read_char'}}();
 		if ( $text->{term_attribs}->{line_buffer} eq " " ){
@@ -54,29 +61,29 @@ sub check_for_spacebar_hit {
 		}
 	});
 }
-sub detect_spacebar {
-	$engine->{events}->{stdin} = undef; # clean up after get_edit_mark()
-	check_for_spacebar_hit() # if $config->{press_space_to_start};
-}
 
 sub throw {
 	logsub("&throw");
-	pager2(@_)
+	pager3(@_)
 }
 sub pager2 {
 	logsub("&pager2");
 	pager(join "", @_)
 }
+sub pager3 { map { my $s = $_; chomp $s; say $s} @_ }
+	
 sub pager {
 	logsub("&pager");
 	my @output = @_;
-	my ($screen_lines, $columns) = $text->{term}->get_screen_size();
+	my ($screen_lines, $columns) =
+	$text->{term} ? $text->{term}->get_screen_size() : (); 
 	my $line_count = 0;
 	map{ $line_count += $_ =~ tr(\n)(\n) } @output;
 	if 
 	( 
 		(ref $ui) =~ /Text/  # pager interferes with GUI
 		and $config->{use_pager} 
+		and ! $config->{opts}->{T}
 		and $line_count > $screen_lines - 2
 	) { 
 		my $fh = File::Temp->new();
@@ -88,6 +95,24 @@ sub pager {
 	}
 	print "\n\n";
 }
+
+sub mandatory_pager {
+	logsub("&mandatory_pager");
+	my @output = @_;
+	if 
+	( 
+		(ref $ui) =~ /Text/  # pager interferes with GUI
+		and $config->{use_pager} 
+	) { 
+		my $fh = File::Temp->new();
+		my $fname = $fh->filename;
+		print $fh @output;
+		file_pager($fname);
+	} else {
+		print @output;
+	}
+	print "\n\n";
+} 
 sub file_pager {
 	logsub("&file_pager");
 	my $fname = shift;
@@ -125,19 +150,6 @@ sub get_ecasound_iam_keywords {
 				grep{ ! $reserved{$_} } split /[\s,]/, eval_iam('int-cmd-list');
 }
 
-sub process_line {
-	logsub("&process_line");
-	my ($user_input) = @_;
-	logit(__LINE__,'Audio::Nama::Terminal','debug',"user input: $user_input");
-	if (defined $user_input and $user_input !~ /^\s*$/) {
-		$text->{term}->addhistory($user_input) 
-			unless $user_input eq $text->{previous_cmd};
-		$text->{previous_cmd} = $user_input;
-		command_process( $user_input );
-		reconfigure_engine();
-		revise_prompt();
-	}
-}
 sub load_keywords {
 	@{$text->{keywords}} = keys %{$text->{commands}};
 	push @{$text->{keywords}}, grep{$_} map{split " ", $text->{commands}->{$_}->{short}} @{$text->{keywords}};
