@@ -1,56 +1,10 @@
 # ---------- Track -----------
 #
-# give all classes (packages) access to global vars 
-
 package Audio::Nama;
-our (
-	$ui,
-	$mode,
-	$file,
-	$graph,
-	$setup,
-	$config,
-	$jack,
-	$fx,
-	$fx_cache,
-	$engine,
-	$text,
-	$gui,
-	$midi,
-	$help,
-	$mastering,
-	$project,
-	$this_track,
-	$this_bus,
-	$this_op,
-	$this_param,
-	$this_mark,
-	$this_edit,
-	$prompt,
-	%tn,
-	%ti,
-	%bn,
-	$debug,
-	$debug2,
-	@config_vars,
-	@persistent_vars,
-	@new_persistent_vars,
-	@project_config_vars,
-	@tracks_data,
-	@bus_data,
-	@groups_data,
-	@marks_data,
-	@fade_data,
-	@edit_data,
-	@inserts_data,
-	@global_effect_chain_vars,
-	@global_effect_chain_data,
-	@project_effect_chain_data,
-	$this_track_name
-);
 {
 package Audio::Nama::Track;
-use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg logsub);
 
 # Objects belonging to Track and its subclasses
 # have a 'class' field that is set when the 
@@ -84,34 +38,74 @@ use Audio::Nama::Object qw(
 					n   			
 					name
 					group 			
+
 					rw				
 					version         
 					width			
+
 					ops 			
 					vol				
 					pan				
 					fader			
 					latency_op		
 					offset			
+									
+									
+
 					old_vol_level	
 					old_pan_level
+
 					playat			
 					region_start	
 					region_end
 					modifiers		
 					looping			
+
 					hide			
+
 					source_id		
 					source_type		
+
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+					
+
 					send_id			
 					send_type
+
 					target			
 					project			
+					
 					rec_defeat		
+
 					effect_chain_stack 
-					cache_map		
+
 					comment			
 					version_comment 
+
+					forbid_user_ops	
+
+					
+					
+				    
+					
+					
+					
+					
 					current_edit    
 
 );
@@ -191,9 +185,6 @@ sub new {
 					send_type 		=> undef,
 					send_id   		=> undef,
 					effect_chain_stack => [],
-					cache_map 		=> {},
-					current_edit 	=> {},
-					version_comment => {},
 
 					@_ 			}, $class;
 
@@ -205,7 +196,7 @@ sub new {
 	Audio::Nama::add_pan_control($n);
 	Audio::Nama::add_volume_control($n);
 
-	$this_track = $object;
+	$Audio::Nama::this_track = $object;
 	$object;
 	
 }
@@ -256,7 +247,7 @@ sub current_wav {
 		my $filename = $track->targets->{ $track->monitor_version } ;
 		$filename
 	} else {
-		logit(__LINE__,'Audio::Nama::Track','debug', "track ", $track->name, ": no current version") ;
+		logpkg(__FILE__,__LINE__,'debug', "track ", $track->name, ": no current version") ;
 		undef; 
 	}
 }
@@ -264,7 +255,7 @@ sub current_wav {
 sub current_version {	
 	my $track = shift;
 	my $status = $track->rec_status;
-	#logit(__LINE__,'Audio::Nama::Track','debug', "last: $last status: $status");
+	#logpkg(__FILE__,__LINE__,'debug', "last: $last status: $status");
 
 	# two possible version numbers, depending on REC/MON status
 	
@@ -305,8 +296,8 @@ sub rec_status {
 	my $monitor_version = $track->monitor_version;
 
 	my $bus = $bn{$track->group};
-	#logit(__LINE__,'Audio::Nama::Track','debug', join " ", "bus:",$bus->name, $bus->rw);
-	logit(__LINE__,'Audio::Nama::Track','debug', "track: ", $track->name, ", source: ",
+	#logpkg(__FILE__,__LINE__,'debug', join " ", "bus:",$bus->name, $bus->rw);
+	logpkg(__FILE__,__LINE__,'debug', "track: ", $track->name, ", source: ",
 		$track->source_id, ", monitor version: $monitor_version");
 
 	# first, check for conditions resulting in status 'OFF'
@@ -415,9 +406,17 @@ sub region_is_out_of_bounds {
 
 sub fancy_ops { # returns list 
 	my $track = shift;
-	my @skip = grep {$_} map { $track->$_ } qw(vol pan fader);
+	my @skip = 	grep {fx($_)}  # must have data structure
+				grep {$_} 	   # must be non null
+				map { $track->$_ } qw(vol pan fader latency_op );
+
+	# make a dictionary of ops to exclude
+	# that includes utility ops and their controllers
+	
 	my %skip;
-	map{ $skip{$_}++ } Audio::Nama::expanded_ops_list(@skip);
+
+	map{ $skip{$_}++ } @skip, Audio::Nama::expanded_ops_list(@skip);
+
 	grep{ ! $skip{$_} } @{ $track->ops };
 }
 		
@@ -470,9 +469,9 @@ sub input_path { # signal path, not file path
 	}
 }
 
-### remove and destroy
 
-sub remove_effect_from_track { # doesn't touch $fx->{applied} or $fx->{params} data structures 
+sub remove_effect_from_track { 
+	# doesn't touch $fx->{applied} or $fx->{params} data structures 
 	my $track = shift;
 	my @ids = @_;
 	$track->set(ops => [ grep { my $existing = $_; 
@@ -484,6 +483,11 @@ sub has_insert  { $_[0]->prefader_insert or $_[0]->postfader_insert }
 
 sub prefader_insert { Audio::Nama::Insert::get_id($_[0],'pre') }
 sub postfader_insert { Audio::Nama::Insert::get_id($_[0],'post') }
+sub inserts {  [  # return array ref
+					map{ $Audio::Nama::Insert::by_index{$_} }grep{$_} 
+					map{ Audio::Nama::Insert::get_id($_[0],$_)} qw(pre post) 
+				]
+}
 
 # remove track object and all effects
 
@@ -491,7 +495,7 @@ sub remove {
 	my $track = shift;
 	my $n = $track->n;
 	$ui->remove_track_gui($n); 
- 	$this_track = $ti{Audio::Nama::Track::idx() - 1};
+ 	$Audio::Nama::this_track = $ti{Audio::Nama::Track::idx() - 1};
 	# remove corresponding fades
 	map{ $_->remove } grep { $_->track eq $track->name } values %Audio::Nama::Fade::by_index;
 	# remove effects
@@ -547,9 +551,20 @@ sub set_io {
 		# rec_defeat tracks with 'null' input
 
 		when ('null'){ 
-			$track->set(rec_defeat => 1);
-			say $track->name, ": recording disabled by default for 'null' input.";
-			say "Use 'rec_enable' if necessary";
+
+			if ( $direction eq 'source' ){
+				$track->set(rec_defeat => 1);
+				say $track->name, ": recording disabled by default for 'null' input.";
+				say "Use 'rec_enable' if necessary";
+			}
+		}
+		when ('rtnull'){ 
+
+			if ( $direction eq 'source' ){
+				$track->set(rec_defeat => 1);
+				say $track->name, ": recording disabled by default for 'rtnull' input.";
+				say "Use 'rec_enable' if necessary";
+			}
 		}
 
 		# don't allow user to set JACK I/O unless JACK server is running
@@ -606,21 +621,33 @@ sub send { # command for setting, showing track source
 }
 sub set_source { # called from parser 
 	my $track = shift;
-	my ($source, $type) = @_;
+	my ($source, $type) = @_; # we don't usually get $type passed
 	my $old_source = $track->input_object_text;
 	$track->set_io('source',$source, $type);
 	my $new_source = $track->input_object_text;;
 	my $object = $new_source;
 	if ( $old_source  eq $new_source ){
-		print $track->name, ": input unchanged, $object\n";
+		Audio::Nama::pager2($track->name, ": input unchanged, $object");
 	} else {
-		print $track->name, ": input set to $object\n";
-		# re-enable recording of null-source tracks
-		# TODO: does null source really get recorded?
-		say($track->name, ": record enabled"),
-		$track->set(rec_defeat => 0) if $old_source eq 'null';
+		Audio::Nama::pager2("Track ",$track->name, ": source set to $object");
+		Audio::Nama::pager2("Track ",$track->name, ": record enabled"), 
+			$track->set(rec_defeat => 0) 
+			if transition_from_null($old_source,$new_source)
+			and $track->rec_defeat;
 	}
 }
+{
+my $null_re = /^(rt)?null$/;
+sub transition_from_null {
+	my ($old, $new) = @_;
+	$old =~ /$null_re/ and $new !~ /$null_re/
+}
+sub transition_to_null {
+	my ($old, $new) = @_;
+	$old !~ /$null_re/ and $new =~ /$null_re/
+}
+}
+
 
 sub set_version {
 	my ($track, $n) = @_;
@@ -638,15 +665,20 @@ sub set_version {
 
 sub set_send { # wrapper
 	my ($track, $output) = @_;
-	my $old_send = $track->send;
-	my $new_send = $track->send($output);
+	my $old_send = $track->output_object_text;
+	logpkg(__FILE__,__LINE__,'debug', "send was $old_send");
+	$track->send($output);
+	my $new_send = $track->output_object_text;
+	logpkg(__FILE__,__LINE__,'debug', "send is now $new_send");
 	my $object = $track->output_object_text;
 	if ( $old_send  eq $new_send ){
-		print $track->name, ": send unchanged, ",
-			( $object ?  $object : 'off'), "\n";
+		Audio::Nama::pager2("Track ",$track->name, ": send unchanged, ",
+			( $object ?  $object : 'off'));
 	} else {
-		print $track->name, ": aux output ",
-		($object ? "to $object" : 'is off.'), "\n";
+		Audio::Nama::pager2("Track ",$track->name, ": ", 
+		$object 
+			? "$object is now a send target" 
+			: "send target is turned off.");
 	}
 }
 
@@ -850,7 +882,7 @@ sub import_audio  {
 		#say $cmd;
 		system($cmd) == 0 or say("Ecasound exited with error: ", $?>>8), return;
 	} 
-	Audio::Nama::rememoize() if $config->{opts}->{R}; # usually handled by reconfigure_engine() 
+	Audio::Nama::restart_wav_memoize() if $config->{opts}->{R}; # usually handled by reconfigure_engine() 
 }
 
 sub port_name { $_[0]->target || $_[0]->name } 
@@ -927,9 +959,10 @@ sub adjusted_length {
 
 sub version_comment {
 	my ($track, $v) = @_;
-	my $text   = $track->{version_comment}{$v}{user};
+	return unless $project->{track_version_comments}->{$track->name}{$v};
+	my $text   = $project->{track_version_comments}->{$track->name}{$v}{user};
 	$text .= " " if $text;
-	my $system = $track->{version_comment}{$v}{system};
+	my $system = $project->{track_version_comments}->{$track->name}{$v}{system};
 	$text .= "* $system" if $system;
 	"$v: $text\n" if $text;
 }
@@ -968,16 +1001,55 @@ sub capture_latency {
 	my $io = $track->input_object;
 	return $io->capture_latency if ref $io;
 }
-sub capture_latency {
-	my $track = shift;
-	my $io = $track->input_object;
-	return $io->capture_latency if ref $io;
-}
 sub playback_latency {
 	my $track = shift;
 	my $io = $track->input_object;
-	return $io->capture_latency if ref $io;
+	return $io->playback_latency if ref $io;
 }
+sub sibling_latency {
+	my $track = shift;
+	$setup->{latency}->{sibling}->{$track->name}
+}
+sub sibling_count {
+	my $track = shift;
+	$setup->{latency}->{sibling_count}->{$track->name}
+}
+
+sub set_comment {
+	my ($track, $comment) = @_;
+	$project->{track_comments}->{$track->name} = $comment
+}
+sub comment { $project->{track_comments}->{$_[0]->name} }
+
+sub show_version_comments {
+	my ($t, @v) = @_;
+	return unless @v;
+	Audio::Nama::pager(map{ $t->version_comment($_) } @v);
+}
+sub add_version_comment {
+	my ($t,$v,$text) = @_;
+	$t->targets->{$v} or say("$v: no such version"), return;	
+	$project->{track_version_comments}->{$t->name}{$v}{user} = $text;
+	$t->version_comment($v);
+}
+sub add_system_version_comment {
+	my ($t,$v,$text) = @_;
+	$t->targets->{$v} or say("$v: no such version"), return;	
+	$project->{track_version_comments}{$t->name}{$v}{system} = $text;
+	$t->version_comment($v);
+}
+sub remove_version_comment {
+	my ($t,$v) = @_;
+	$t->targets->{$v} or say("$v: no such version"), return;	
+	delete $project->{track_version_comments}{$t->name}{$v}{user};
+	$t->version_comment($v) || "$v: [comment deleted]\n";
+}
+sub remove_system_version_comment {
+	my ($t,$v) = @_;
+	delete $project->{track_version_comments}{$t->name}{$v}{system} if
+		$project->{track_version_comments}{$t->name}{$v}
+}
+
 } # end package
 
 # subclasses
@@ -985,7 +1057,8 @@ sub playback_latency {
 
 {
 package Audio::Nama::SimpleTrack; # used for Master track
-use Modern::Perl; use Carp; use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Modern::Perl; use Carp; use Audio::Nama::Log qw(logpkg);
 no warnings qw(uninitialized redefine);
 our @ISA = 'Audio::Nama::Track';
 sub rec_status { $_[0]->rw ne 'OFF' ? 'REC' : 'OFF' }
@@ -995,7 +1068,8 @@ sub unbusify {}
 }
 {
 package Audio::Nama::MasteringTrack; # used for mastering chains 
-use Modern::Perl; use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Modern::Perl; use Audio::Nama::Log qw(logpkg);
 no warnings qw(uninitialized redefine);
 our @ISA = 'Audio::Nama::SimpleTrack';
 
@@ -1009,7 +1083,8 @@ sub version {0}
 }
 {
 package Audio::Nama::SlaveTrack; # for instrument monitor bus
-use Modern::Perl; use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Modern::Perl; use Audio::Nama::Log qw(logpkg);
 no warnings qw(uninitialized redefine);
 our @ISA = 'Audio::Nama::Track';
 sub width { $tn{$_[0]->target}->width }
@@ -1025,7 +1100,8 @@ sub dir { $tn{$_[0]->target}->dir }
 }
 {
 package Audio::Nama::CacheRecTrack; # for graph generation
-use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
 our @ISA = qw(Audio::Nama::SlaveTrack);
 sub current_version {
 	my $track = shift;
@@ -1043,13 +1119,14 @@ sub full_path { my $track = shift; Audio::Nama::join_path( $track->dir, $track->
 }
 {
 package Audio::Nama::MixDownTrack; 
-use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
 our @ISA = qw(Audio::Nama::Track);
 sub current_version {	
 	my $track = shift;
 	my $last = $track->last;
 	my $status = $track->rec_status;
-	#logit(__LINE__,'Audio::Nama::Track','debug', "last: $last status: $status");
+	#logpkg(__FILE__,__LINE__,'debug', "last: $last status: $status");
 	if 	($status eq 'REC'){ return ++$last}
 	elsif ( $status eq 'MON'){ return $track->monitor_version } 
 	else { return 0 }
@@ -1059,15 +1136,17 @@ sub rec_status {
 	return 'REC' if $track->rw eq 'REC';
 	Audio::Nama::Track::rec_status($track);
 }
+sub forbid_user_ops { 1 }
 }
 {
 package Audio::Nama::EditTrack; use Carp qw(carp cluck);
-use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
 our @ISA = 'Audio::Nama::Track';
 our $AUTOLOAD;
 sub AUTOLOAD {
 	my $self = shift;
-	logit(__LINE__,'Audio::Nama::Track','debug', $self->name, ": args @_");
+	logpkg(__FILE__,__LINE__,'debug', $self->name, ": args @_");
     # get tail of method call
     my ($call) = $AUTOLOAD =~ /([^:]+)$/;
 	$Audio::Nama::Edit::by_name{$self->name}->$call(@_);
@@ -1077,26 +1156,28 @@ sub current_version {
 	my $track = shift;
 	my $last = $track->last;
 	my $status = $track->rec_status;
-	#logit(__LINE__,'Audio::Nama::Track','debug', "last: $last status: $status");
+	#logpkg(__FILE__,__LINE__,'debug', "last: $last status: $status");
 	if 	($status eq 'REC' and ! $track->rec_defeat){ return ++$last}
 	elsif ( $status eq 'MON'){ return $track->monitor_version } 
 	else { return 0 }
 }
 sub playat_time {
-	logit(__LINE__,'Audio::Nama::Track','logcluck',$_[0]->name . "->playat_time");
+	logpkg(__FILE__,__LINE__,'logcluck',$_[0]->name . "->playat_time");
 	$_[0]->play_start_time
 }
 }
 {
 package Audio::Nama::VersionTrack;
-use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
 our @ISA ='Audio::Nama::Track';
 sub set_version {}
 sub versions { [$_[0]->version] }
 }
 {
 package Audio::Nama::MixTrack;
-use Audio::Nama::Log qw(logit);
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
 our @ISA ='Audio::Nama::Track';
 # as a mix track, I have no sources of my own
 # when status is REC
@@ -1113,6 +1194,7 @@ sub input_path {
 {
 package Audio::Nama;
 use Modern::Perl;
+use Audio::Nama::Effects qw(:all);
 
 # usual track
 
@@ -1124,22 +1206,25 @@ sub add_track {
 	my %vals = (name => $name, @params);
 	my $class = $vals{class} // 'Audio::Nama::Track';
 	{ no warnings 'uninitialized';	
-	logit(__LINE__,'Audio::Nama::Track','debug', "name: $name, ch_r: $gui->{_chr}, ch_m: $gui->{_chm}");
+	logpkg(__FILE__,__LINE__,'debug', "name: $name, ch_r: $gui->{_chr}, ch_m: $gui->{_chm}");
 	}	
 	say("$name: track name already in use. Skipping."), return 
 		if $tn{$name};
 	say("$name: reserved track name. Skipping"), return
 	 	if grep $name eq $_, @{$mastering->{track_names}}; 
 
+	# in order to increment serially
+
+	Audio::Nama::ChainSetup::remove_temporary_tracks();
+
 	my $track = $class->new(%vals);
 	return if ! $track; 
-	$this_track = $track;
-	logit(__LINE__,'Audio::Nama::Track','debug', "ref new track: ", ref $track); 
+	logpkg(__FILE__,__LINE__,'debug', "ref new track: ", ref $track); 
 	$track->source($gui->{_chr}) if $gui->{_chr};
 #		$track->send($gui->{_chm}) if $gui->{_chm};
 
 	my $bus = $bn{$track->group}; 
-	command_process('for mon; mon') if $mode->{preview} and $bus->rw eq 'MON';
+	process_command('for mon; mon') if $mode->{preview} and $bus->rw eq 'MON';
 	$bus->set(rw => 'REC') unless $track->target; # not if is alias
 
 	# normal tracks default to 'REC'
@@ -1151,7 +1236,7 @@ sub add_track {
 
 	set_current_bus();
 	$ui->track_gui($track->n);
-	logit(__LINE__,'Audio::Nama::Track','debug', "Added new track!\n", sub{$track->dump});
+	logpkg(__FILE__,__LINE__,'debug', "Added new track!\n", sub{$track->dump});
 	$track;
 }
 
@@ -1233,10 +1318,10 @@ sub add_volume_control {
 	my $n = shift;
 	return unless need_vol_pan($ti{$n}->name, "vol");
 	
-	my $vol_id = cop_add({
+	my $vol_id = effect_init({
 				chain => $n, 
 				type => $config->{volume_control_operator},
-				cop_id => $ti{$n}->vol, # often undefined
+				effect_id => $ti{$n}->vol, # often undefined
 				});
 	
 	$ti{$n}->set(vol => $vol_id);  # save the id for next time
@@ -1246,16 +1331,15 @@ sub add_pan_control {
 	my $n = shift;
 	return unless need_vol_pan($ti{$n}->name, "pan");
 
-	my $pan_id = cop_add({
+	my $pan_id = effect_init({
 				chain => $n, 
 				type => 'epp',
-				cop_id => $ti{$n}->pan, # often undefined
+				effect_id => $ti{$n}->pan, # often undefined
 				});
 	
 	$ti{$n}->set(pan => $pan_id);  # save the id for next time
 	$pan_id;
 }
-
 } # end package
 
 1;
